{
  Fnugry CloneControl Component
  Version 1.0.1.2
  Copyright (C) 1997 Gleb Yourchenko


  E-Mail: eip__@hotmail.com
  Please specify 'TFnugryCloneControl' in the subject string.



  TFnugryCloneControl is a simple tool that allows developer
  to control a number of application instances user can run.

}

unit FngUniInst;

interface
uses
  Windows, SysUtils, Classes;

type

  ECloneControlError = class(Exception);

  TCloneControlAction = (clcNone, clcMessage, clcCustom);
  TTerminationStyle = (tsAppTerminate, tsHalt, tsCustom);

  TFnugryCloneControl = class(TComponent)
  private
    FHandle           :THandle;
    FMappingName      :String;
    FMaxInstance      :Integer;
    FAction           :TCloneControlAction;
    FTerminationStyle :TTerminationStyle;
    FOnAction         :TNotifyEvent;
    FOnTerminate      :TNotifyEvent;
    procedure Initialize;
    procedure Finalize;
    procedure SetMaxInstance(Value :Integer);
    function GetInstanceCount :Integer;
    function CreateMapping :THandle;
    function MapData :Pointer;
  protected
    procedure Check(fCond :Boolean; const Msg :String);
    procedure Loaded; override;
    procedure DoAction; virtual;
    procedure DoTerminate; virtual;
    property MappingName :String
      read FMappingName;
  public
    constructor Create(AOwner :TCOmponent); override;
    destructor Destroy; override;
    property InstanceCount :Integer
      read GetInstanceCount;
  published
    property Action :TCloneControlAction
      read FAction write FAction;
    property TerminationStyle :TTerminationStyle
      read FTerminationStyle write FTerminationStyle;
    property MaxInstance :Integer
      read FMaxInstance write SetMaxInstance;
    property OnAction :TNotifyEvent
      read FOnAction write FOnAction;
    property OnTerminate :TNotifyEvent
      read FOnTerminate write FOnTerminate;
  end;

procedure Register;


implementation
uses
  Dialogs, Forms;

procedure Register;
begin
  RegisterCOmponents('Fnugry Tools', [TFnugryCloneControl]);
end;

const

  MaXINSTANCE_MIN      = 1;
  MaXINSTANCE_MAX      = $FFF0;

  SErrMultiInstance    = 'Only one instance of CloneControl allowed';
  SNotifyMsg           = 'Only %d instance(s) of application is allowed';
  SFmtMappingName      = 'Fnugry.CloneControl.%s';

type

  PInstInfo = ^TInstInfo;
  TInstInfo = record
    iInstCount :Integer;
  end;



{ TCloneControl }

var CloneControlInstance :TFnugryCloneControl = Nil;

constructor TFnugryCloneControl.Create(AOwner :TCOmponent);
begin
  Check(CloneControlInstance <> Nil, SErrMultiInstance);
  inherited CReate(AOwner);
  FAction := clcMessage;
  FTerminationStyle := tsAPpTerminate;
  FMaxInstance := 1;
  FMappingName := format(SFmtMappingName, [ExtractFileName(Application.ExeName)]);
  CloneControlInstance := Self;
end;


destructor TFnugryCloneControl.Destroy;
begin
  Finalize;
  inherited Destroy;
  CloneControlInstance := Nil;
end;

procedure TFnugryCloneControl.Check(fCond :Boolean; const Msg :String);
begin
  if fCond then raise ECloneControlError.Create(Msg);
end;

procedure TFnugryCloneControl.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then Initialize;
end;

procedure TFnugryCloneControl.SetMaxInstance(Value :Integer);
begin
  if Value < MaXINSTANCE_MIN
    then Value := MAXINSTANCE_MIN
    else
      if Value > MAXINSTANCE_MAX
        then Value := MAXINSTANCE_MAX;
  FMaxInstance := Value;
end;


procedure TFnugryCloneControl.DoAction;
begin
  case Action of
    clcNone    :;
    clcMessage :MessageDlg(format(SNotifyMsg, [FMaxINstance]),
      mtError, [mbOk], 0);
    clcCustom  : if assigned(FOnAction) then FOnAction(Self);
  end;
end;

procedure TFnugryCloneControl.DoTerminate;
begin
  case TerminationStyle of
    tsAppTerminate : Application.Terminate;
    tsHalt : halt;
    tsCustom : if assigned(FOnTerminate) then FOnTerminate(Self);
  end;
end;

procedure TFnugryCloneControl.Initialize;
var
  Info  :PInstInfo;
begin
  FHandle := CreateMapping;
  if FHandle <> 0 then
    begin
      if GetLastError = ERROR_ALREADY_EXISTS then
        begin
          Info := MapData;
          if Info <> Nil then
            if Info^.iInstCount >= FMaxInstance then
              begin
                UnmapViewOfFile(Info);
                CloseHandle(FHandle);
                FHandle := 0;
                DoAction;
                DoTerminate;
              end
            else
              begin
                inc(Info^.iInstCount);
                UnmapViewOfFile(Info);
              end;
        end
      else
       begin
         Info := MapData;
         Info^.iInstCount := 1;
         UnmapViewOfFile(Info);
       end;
    end;
end;

procedure TFnugryCloneControl.Finalize;
var Info :PInstInfo;
begin
  if FHandle <> 0 then
    begin
      Info := MapData;
      if Info <> Nil then
        begin
          Dec(Info^.iInstCount);
          UnmapViewOfFile(Info);
        end;
      CloseHandle(FHandle);
      FHandle := 0;
    end;
end;


function TFnugryCloneControl.GetInstanceCount :Integer;
var Info :PInstInfo;
begin
  result := 0;
  if FHandle <> 0 then
    begin
      Info := MapData;
      if Info <> Nil then
        begin
          result := Info^.iInstCount;
          UnmapViewOfFile(Info);
        end;
    end;
end;

function TFnugryCloneControl.CreateMapping :THandle;
begin
  result := CreateFileMapping(-1, NIL, PAGE_READWRITE,
    0, sizeof(TInstInfo), PChar(MappingName));
end;

function TFnugryCloneControl.MapData :Pointer;
begin
  result := MapViewOfFile(FHandle,
   FILE_MAP_WRITE, 0, 0, sizeof(TInstInfo));
end;

end.
